﻿using System;
using System.Net.Sockets;
using System.Text;

namespace RRHubDemoApp
{
    /// <summary>
    /// Simple C# class to send commands to the RedRatHubCmd application via a socket.
    /// Chris Dodge, Jason Rawlings - RedRat Ltd.
    /// </summary>
    public class Client
    {
        private static Socket socket;
        
        /// <summary>
        /// Opens the socket to RedRatHubCmd.
        /// </summary>
        public void OpenSocket(string host, int port)
        {
            if (socket != null && socket.Connected) return;

            socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            socket.Connect(host, port);
        }

        /// <summary>
        /// Closes the RedRatHubCmd socket.
        /// </summary>
        public void CloseSocket()
        {
            socket.Close();
        }

        /// <summary>
        /// Sends a message to the ReadData() function
        /// </summary>
        public void SendMessage(string message)
        {
            ReadData(message);
        }

        /// <summary>
        /// Reads data back from RedRatHub.
        /// </summary>
        public string ReadData(string message)
        {
            if (socket == null || !socket.Connected)
            {
                Console.WriteLine("\tSocket has not been opened. Call 'OpenSocket()' first.");
                return null;
            }

            // Send message
            socket.Send(Encoding.UTF8.GetBytes(message + "\n"));
            var received = "";

            // Check response. This is either a single line, e.g. "OK\n", or a multi-line response with 
            // '{' and '}' start/end delimiters.
            while (true)
            {
                var receivedBytesFull = new byte[256];

                // Receive data
                var receivedLength = socket.Receive(receivedBytesFull);
                var receivedBytes = new byte[receivedLength];
                Array.Copy(receivedBytesFull, receivedBytes, receivedLength);
                received += Encoding.UTF8.GetString(receivedBytes);

                if (CheckEom(received)) return received;
            }
        }

        /// <summary>
        /// Checks for the end of a message
        /// </summary>
        public bool CheckEom(string message)
        {
            // Multi-line message
            if (message.EndsWith("}"))
            {
                return message.StartsWith("{");
            }

            // Single line message
            return message.EndsWith("\n");
        }
    }
}
